@@ -28,13 +28,12 @@
  * MT safe
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+
 #include "glib.h"
 
 
