@@ -56,6 +56,7 @@ static gulong g_thread_min_stack_size = 0;
 #define G_MUTEX_SIZE (sizeof (mutex_t))
 
 #define PRIORITY_LOW_VALUE 0
+#define PRIORITY_NORMAL_VALUE 50
 #define PRIORITY_URGENT_VALUE 127
 
 #define HAVE_G_THREAD_IMPL_INIT
@@ -63,6 +64,8 @@ static void
 g_thread_impl_init()
 {
   g_thread_min_stack_size = thr_min_stack();
+  /* The default priority on Solaris is 0. Set it to something sane */
+  solaris_check_for_error (thr_setprio (thr_self (), PRIORITY_NORMAL_VALUE));
 }
 
 static GMutex *
@@ -208,7 +211,7 @@ g_thread_create_solaris_impl (GThreadFunc thread_func,
   g_return_if_fail (priority <= G_THREAD_PRIORITY_URGENT);
   
   if (stack_size)
-  stack_size = MAX (g_thread_min_stack_size, stack_size);
+    stack_size = MAX (g_thread_min_stack_size, stack_size);
   
   ret = thr_create (NULL, stack_size, (void* (*)(void*))thread_func,
 		    arg, flags, thread);
